# -*- coding: utf-8 -*-
"""
# @Time    : 2022/3/3 15:43
# @Author  : bruce
# @desc    :
"""

import os
import logging
from enum import Enum

Log_Format = '%(asctime)s [%(levelname)s] [%(filename)s:%(lineno)d] [%(thread)d] - %(message)s'


class LogLevel(Enum):
    CRITICAL = 'CRITICAL'
    FATAL = 'FATAL'
    ERROR = 'ERROR'
    WARNING = 'WARNING'
    INFO = 'INFO'
    DEBUG = 'DEBUG'
    NOTSET = 'NOTSET'


_logLevelName = ['CRITICAL', 'FATAL', 'ERROR', 'WARN', 'WARNING', 'INFO', 'DEBUG', 'NOTSET']


class Log:
    def __init__(self, logger_name=None):
        self.default_log_level = "DEBUG"
        self.logger = logging.getLogger(logger_name)
        # default console format
        self._console_format = Log_Format
        # default file format
        self._file_format = Log_Format
        # set log level
        self._file_log_level = LogLevel.DEBUG.value
        self._console_log_level = LogLevel.INFO.value

    def init_logger(self, file_log_path=None):
        if file_log_path:
            # make dir
            dir_path = os.path.dirname(file_log_path)
            if not os.path.exists(dir_path):
                os.makedirs(dir_path)
            # file handler
            file_handler = self.build_file_handler(file_log_path)
            self.logger.addHandler(file_handler)

        # console handler
        console_handler = self.build_console_handler()
        self.logger.addHandler(console_handler)

        self.logger.setLevel(self.default_log_level)
        return self.logger

    @property
    def console_format(self):
        """
        log format
        :return:
        """
        return self._console_format

    @console_format.setter
    def console_format(self, log_format: str):
        """
        :param log_format:
        :return:
        """
        self._console_format = log_format

    @property
    def file_format(self):
        """
        log format
        :return:
        """
        return self._file_format

    @file_format.setter
    def file_format(self, log_format: str):
        """
        :param log_format:
        :return:
        """
        self._file_format = log_format

    @property
    def file_log_level(self):
        """
        :return:
        """
        return self._file_log_level

    @file_log_level.setter
    def file_log_level(self, level: str):
        """
        :param level: file log level in ['CRITICAL' | 'FATAL' | 'ERROR' | 'WARN' | 'WARNING' | 'INFO' | 'DEBUG']
        :return:
        """
        level = level.upper()
        if level in _logLevelName:
            self._file_log_level = level
        else:
            self.logger.error("Set log level error: unknown level name. Level name must in: "
                              "['CRITICAL' | 'FATAL' | 'ERROR' | 'WARN' | 'WARNING' | 'INFO' | 'DEBUG']")

    @property
    def console_log_level(self):
        """
        :return:
        """
        return self._console_log_level

    @console_log_level.setter
    def console_log_level(self, level: str):
        """
        :param level: file log level in ['CRITICAL' | 'FATAL' | 'ERROR' | 'WARN' | 'WARNING' | 'INFO' | 'DEBUG']
        :return:
        """
        level = level.upper()
        if level in _logLevelName:
            self._console_log_level = level
        else:
            self.logger.error("Set log level error: unknown level name. Level name must in: "
                              "['CRITICAL' | 'FATAL' | 'ERROR' | 'WARN' | 'WARNING' | 'INFO' | 'DEBUG']")

    def build_file_handler(self, file_log_path):
        """
        build file handler
        :param file_log_path:
        :return:
        """
        formatter = logging.Formatter(self._file_format)
        file_handler = logging.FileHandler(file_log_path, encoding='utf-8')
        file_handler.setLevel(self.file_log_level)          # set level
        file_handler.setFormatter(formatter)                # set format
        return file_handler

    def build_console_handler(self):
        """
        build console handler
        :return:
        """
        formatter = logging.Formatter(self._console_format)
        console_handler = logging.StreamHandler()
        console_handler.setLevel(self.console_log_level)       # set level
        console_handler.setFormatter(formatter)                # set format
        return console_handler


if __name__ == '__main__':
    log = Log()
    log1 = Log()
    print(id(log))
    print(id(log1))
    # log.console_log_level = 'error'
    # log.file_log_level = 'info'
    # logger = log.init_logger(r'D:\PyWorker\ApiTestEz\tests\case\node\report\result')
    # # logger.setLevel('DEBUG')
    # logger.info('1111eeee')



# -*- coding: utf-8 -*-
"""
# @Time    : 2022/3/5 17:58
# @Author  : bruce
# @desc    :
"""
from importlib import import_module

from api_test_ez.ez.decorator import singleton
from api_test_ez.project.settings import default_settings


class BaseSettings(dict):
    """
    Instances of this class behave like dictionaries,
    store with their ``(key, value)`` pairs.
    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)

    def set_module(self, module):
        """
        Store project from a module.

        :param module: the module or the path of the module
        :type module: module object or string
        """
        if isinstance(module, str):
            module = import_module(module)
        for key in dir(module):
            if key.isupper():
                self.set(key, getattr(module, key))

    def set(self, name, value):
        """
        Store a key/value attribute with a given priority.

        :param name: the setting name
        :type name: string

        :param value: the value to associate with the setting
        :type value: any
        """
        self.__setitem__(name, value)


@singleton
class Settings(BaseSettings):
    """
    This object stores ApiTestEz project for the configuration of internal
    components, and can be used for any further customization.
    """
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.set_module(default_settings)


def iter_default_settings():
    """Return the default project as an iterator of (name, value) tuples"""
    for name in dir(default_settings):
        if name.isupper():
            yield name, getattr(default_settings, name)


def overridden_settings(settings):
    """Return a dict of the project that have been overridden"""
    for name, def_value in iter_default_settings():
        value = settings[name]
        if not isinstance(def_value, dict) and value != def_value:
            yield name, value



# -*- coding: utf-8 -*-
"""
# @Time    : 2022/4/26 19:33
# @Author  : bruce
# @desc    :
"""
from api_test_ez.ez import Http
from api_test_ez.ez.decorator.jsonbean import json_bean


class Request(object):

    def __init__(self, http: Http):
        self._http = http
        self._http_data = {}
        self._meta_data = {}
        self._url = None
        self._host = None
        self._path = None
        self._method = None
        self._body = None
        self._owner = None
        self._body_type = None

    def _filter_data(self, request_data):
        if request_data:
            self._url = request_data.pop("url", default=None)
            self._host = request_data.pop("host", default=None)
            self._path = request_data.pop("path", default=None)
            self._body_type = request_data.pop("body_type", default="data")
            if self._url is None and self._host:
                if not self._host.startswith('http'):
                    self._host = f'http://{self._host}'
                if self._path:
                    self._url = f'{self._host}{self._path}' \
                        if self._path.startswith('/') \
                        else f'{self._host}/{self._path}'
                else:
                    self._url = self._host

            self._method = request_data.pop("method")
            self._body = request_data.pop("body", default=None)
            # http
            self._http_data = {
                "headers": request_data.pop("headers"),
                "timeout": request_data.pop("timeout"),
                "cookies": request_data.pop("cookies"),
                "retry": request_data.pop("retry"),
                "proxies": request_data.pop("proxies"),
                "allow_redirects": request_data.pop("allow_redirects"),
                "verify": request_data.pop("verify"),
            }

            # meta
            self._meta_data = request_data

    def set(self, request_data):
        self._filter_data(request_data)
        for key, value in self._http_data.items():
            if value:
                if hasattr(self._http, key):
                    setattr(self._http, key, value)

    @property
    def owner(self):
        return self._owner

    @owner.setter
    def owner(self, value):
        self._owner = value

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, value):
        self._url = value

    @property
    def host(self):
        return self._host

    @host.setter
    def host(self, value):
        self._host = value

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, value):
        self._path = value

    @property
    def method(self):
        return self._method

    @method.setter
    def method(self, value):
        self._method = value

    @property
    def body(self):
        return self._body

    @body.setter
    def body(self, value):
        self._body = value

    @property
    def body_type(self):
        return self._body_type

    @body_type.setter
    def body_type(self, value):
        self._body_type = value

    @property
    def http(self):
        return self._http

    @property
    def meta(self):
        return RequestMetaData(self._meta_data)

    def __str__(self):
        return f'<{self.__class__.__name__}> [{self.owner}]:\n' \
               f'url: {self._url!r}\n' \
               f'method: {self._method!r}\n' \
               f'body: {self._body!r}\n' \
               f'http_data: {self._http_data!r}\n' \
               f'meta: {self.meta.data!r}'

    __repr__ = __str__


class RequestMetaData:

    def __init__(self, meta_dict):
        self._meta_dict = meta_dict

    @property
    @json_bean
    def bean(self):
        return self._meta_dict.to_dict()

    @property
    def data(self):
        return self._meta_dict.to_dict()


if __name__ == '__main__':
    http = Http()
    request = Request(http)
    request.set(
        {
            "url": 0,
            "method": 11,
            "body": 12,
            "headers": 1,
            "timeout": 2,
            "cookies": 3,
            "retry": 4,
            "proxies": 5,
            "allow_redirects": 6,
            "verify": 7,
        }
    )



# -*- coding: utf-8 -*-
"""
# @Time    : 2022/3/3 15:43
# @Author  : bruce
# @desc    :
"""


class JsonBean(type):
    """
    # JsonBean类，具有以下特性：
    # 1、将json实例化为对象调用（OOP）
    # 2、为对象属性提供只读功能
    """
    def __new__(mcs, *args, **kwargs):
        if len(args) > 0:
            json = args[0]
            if not isinstance(json, dict):
                raise Exception('Json required!')
            else:
                for k, v in json.items():
                    if isinstance(v, dict):
                        json.update({k: JsonBean(v)})

                    elif isinstance(v, list):
                        new_v = []
                        for _v in v:
                            if isinstance(_v, dict):
                                new_v.append(JsonBean(_v))
                            else:
                                new_v.append(_v)
                        json.update({k: new_v})

                    else:
                        json.update({k: v})

                return super(JsonBean, mcs).__new__(mcs, mcs.__name__, (JsonBean,), json)

    def __getattr__(self, item):
        if item not in self.__dict__:
            return None

    def __getattribute__(self, item):

        return object.__getattribute__(self, item)

    def __setattr__(self, key, value):
        raise Exception("It is read only!")


def json_bean(func):
    """
    JsonBean装饰器，将json数据转化为JsonBean对象
    :param func:
    :return:
    """
    def wrapper(*args, **kwargs):
        json = func(*args, **kwargs)
        return JsonBean(json)
    return wrapper



# -*- coding: utf-8 -*-
"""
# @Time    : 2022/3/5 19:31
# @Author  : bruce
# @desc    :
"""
import ast
import os
import sys

if sys.version_info[:2] >= (3, 8):
    from collections.abc import MutableMapping
else:
    from collections import MutableMapping
from api_test_ez.ez import get_config

CONFIG_PRIORITIES = {
    'default': 0,
    'project': 10,
    'module': 20,
    'package': 30,
    'case': 40,
    'command': 50,
}


def get_config_priority(priority):
    """
    Small helper function that looks up a given string priority
    """
    return CONFIG_PRIORITIES[priority]


class BaseConfigs(MutableMapping):
    """
    Instances of this class behave like dictionaries,
    store with their ``(key, value)`` pairs.
    but values will store as a list order by ``priority``.
    when you want ``get`` value, it will return the value of the highest priority (maybe now is enough).
    """
    def __init__(self, *args, **kwargs):

        self.attributes = {}
        self.update(*args, **kwargs)

    def __delitem__(self, name) -> None:
        del self.attributes[name]

    def __getitem__(self, name):
        if name not in self:
            return None
        return self.attributes[name].value

    def __len__(self) -> int:
        return len(self.attributes)

    def __iter__(self):
        return iter(self.attributes)

    def __setitem__(self, name, value):
        self.set(name, value)

    def __contains__(self, name):
        return name in self.attributes

    def set_config(self, config_path, priority='default'):
        """
        Store project from a module.

        :param config_path: the config file path
        :type config_path: string

        :param priority: the priority of the configs.
        :type priority: string
        """
        cfg = get_config(config_path)
        for title, selection in cfg.items():
            for key, value in selection.items():
                try:
                    value = ast.literal_eval(value)
                except SyntaxError:
                    pass
                except ValueError:
                    pass
                self.set(key.lower(), value, priority=priority)

    def set(self, name, value, priority='default'):
        """
        Store a key/value attribute with a given priority.

        :param name: the setting name
        :type name: string

        :param value: the value to associate with the setting
        :type value: any

        :param priority: the priority of the setting. Should be a key of
            CONFIG_PRIORITIES or an integer
        :type priority: string or int
        """
        priority = get_config_priority(priority)
        if name not in self:
            if isinstance(value, ConfigAttribute):
                self.attributes[name] = value
            else:
                self.attributes[name] = ConfigAttribute(value, priority)
        else:
            self.attributes[name].set(value, priority)

    def get(self, name, default=None):
        """
        Get a config value without affecting its original type.

        :param name: the setting name
        :type name: string

        :param default: the value to return if no setting is found
        :type default: any
        """
        return self[name] if self[name] is not None else default

    def pop(self, name, default=None):
        """
        Pop a config value from `name` and delete key in config.
        :param name:
        :param default:
        :return:
        """
        value = self[name] if self[name] is not None else default
        if self.attributes.get(name):
            del self.attributes[name]
        return value

    def to_dict(self):
        """
        Change `Config` to `dict`.
        :return:
        """
        return {k: v for k, v in self.items()}


class ConfigAttribute(object):
    """
    Class for storing data related to configs attributes.

    This class is intended for internal usage, you should try Configs class
    for setting configurations, not this one.
    """
    def __init__(self, value, priority):
        self.value = value
        self.priority = priority

    def set(self, value, priority):
        """Sets value if priority is higher or equal than current priority."""
        if priority >= self.priority:
            self.value = value
            self.priority = priority

    def __str__(self):
        return "<ConfigAttribute value={self.value!r} " \
               "priority={self.priority}>".format(self=self)

    __repr__ = __str__


class Configs(BaseConfigs):
    """
    This object stores ApiTestEz project for the configuration of internal
    components, and can be used for any further customization.
    """
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.set_config(os.path.join(os.path.dirname(__file__), "default_configs.cfg"))



# -*- coding: utf-8 -*-
"""
# @Time    : 2022/3/3 15:11
# @Author  : bruce
# @desc    :
"""
from api_test_ez.ez.config import get_config
from api_test_ez.ez.http.http import Http
from api_test_ez.ez.log.log import Log



# -*- coding: utf-8 -*-
"""
# @Time    : 2022/6/9 19:30
# @Author  : bruce
# @desc    :
"""
import re
from abc import abstractmethod, ABC

from api_test_ez.ez.serialize.errors import ValidationError


__all__ = ["StringField", "IntegerField", "ListField", "DynamicListField"]


validator_funcs = [
    'should_be', 'should_in', 'should_contain', 'should_like',
    # For list field
    'count_should_be', 'count_should_gt', 'count_should_lt',
    'count_should_gte', 'count_should_lte',
    # For list members
    'members_should_contain_model'
]


class BaseField(ABC):
    field_type = None

    def __init__(self, required=False, null=False, **kwargs):
        self.required = required
        self.null = null
        self.name = None

        # Discover any validator function, which start with `should_`.
        # Store in `validator_funcs` dict.
        self.validate_funcs = {}
        for key, value in kwargs.items():
            if key in validator_funcs:
                self.validate_funcs.update({key: value})

    def __set_name__(self, owner, name):
        self.name = name

    def __get__(self, instance, owner):
        # return getattr(instance.__fields_mapping__, self.validated_name, None)
        return instance.__fields_mapping__.get(self.name)

    def __set__(self, instance, value):
        # return maybe `errors` or `value`.
        value = self.validate(value)
        instance.__fields_mapping__[self.name] = value

    def error(self, message="", errors=None, field_name=None):
        """Raise a ValidationError."""
        field_name = field_name if field_name else self.name
        return ValidationError(message, errors=errors, field_name=field_name)

    def _func_should_be(self, value, expect_value):
        if value != expect_value:
            return self.error("[Lv2] %s should be %s but %s found." % (self.__class__.__name__, expect_value, value))
        return value

    def _func_should_in(self, value, expect_value):
        if value not in expect_value:
            return self.error("[Lv2] %s %s does not in %s." % (self.__class__.__name__, value, expect_value))
        return value

    def _func_should_contain(self, value, expect_value):
        if expect_value not in value:
            return self.error("[Lv2] %s %s does not contain %s." % (self.__class__.__name__, value, expect_value))
        return value

    @staticmethod
    def _validation_is_error(value):
        if "ValidationError" in str(value):
            return True
        return False

    def validate(self, value):
        """Perform validation on a value."""
        # Lv1 validate
        # If validate fail, just return value.
        if self._validation_is_error(value):
            return value

        if self.field_type is not None:
            value = self._validate_types(value, self.field_type)

        # If validate fail, just return value.
        if self._validation_is_error(value):
            return value

        # Lv2 validate
        value = self._validate_should_funcs(value)

        return value

    def _validate_should_funcs(self, value):
        # call `func_should_`.
        if len(self.validate_funcs) > 0:
            for should, expect_value in self.validate_funcs.items():
                func = getattr(self, "_func_%s" % should)
                value = func(value, expect_value)
        return value

    def _validate_types(self, value, field_type):
        if not isinstance(value, field_type):
            return self.error(f"[Lv1] {self.__class__.__name__} only accepts {self.field_type} values. "
                              f"but {value!r} found.")
        return value


class StringField(BaseField):

    __slots__ = ('should_be', 'should_in', 'should_contain', 'should_like')

    field_type = str

    def _func_should_like(self, value, expect_regex):
        res = re.search(expect_regex, value)
        if res:
            return value
        else:
            return self.error(f"[Lv2] {self.__class__.__name__} {value!r} does not match {expect_regex!r}.")


class IntegerField(BaseField):

    __slots__ = ('should_be', 'should_in', 'should_contain', 'should_like')

    field_type = int


class ListField(BaseField):

    __slots__ = ('should_be', 'should_in', 'should_contain', 'should_not_contain',
                 # For list field
                 'count_should_be', 'count_should_gt', 'count_should_lt',
                 'count_should_gte', 'count_should_lte', 'should_no_duplicates'
                 # For list members
                 'members_should_contain_model'
                 )

    field_type = list

    def __init__(self, *fields, **kwargs):
        self.fields = fields
        super().__init__(**kwargs)

    def _func_should_contain(self, value, expect_value):
        if isinstance(expect_value, list):
            if len(set(value).intersection(set(expect_value))) != len(expect_value):
                return self.error("[Lv2] %s %s does not contain %s." % (self.__class__.__name__, value, expect_value))
        else:
            if expect_value not in value:
                return self.error("[Lv2] %s %s does not contain %s." % (self.__class__.__name__, value, expect_value))
        return value

    def _func_should_no_duplicates(self, value, expect_value=True):
        # bcz dict can not be `set`, loop to validate.
        if isinstance(expect_value, bool):
            new_list = []
            for v in value:
                if v not in new_list:
                    new_list.append(v)
            if len(new_list) != len(value):
                return self.error("[Lv2] %s %s has duplicate data." % (self.__class__.__name__, value))
        else:
            return self.error(f"[Lv2] `should_no_duplicates` only accepts bool params, but {expect_value!r} found.")

    def _func_should_not_contain(self, value, expect_value):
        if isinstance(expect_value, list):
            if len(set(value).intersection(set(expect_value))) > 0:
                return self.error("[Lv2] %s %s should not contain %s." % (self.__class__.__name__, value, expect_value))
        else:
            if expect_value in value:
                return self.error("[Lv2] %s %s should not contain %s." % (self.__class__.__name__, value, expect_value))
        return value

    def _func_should_in(self, value, expect_value):
        if isinstance(expect_value, list):
            if len(set(value).intersection(set(expect_value))) != len(value):
                return self.error("[Lv2] %s %s does not in %s." % (self.__class__.__name__, value, expect_value))
            return value
        else:
            return self.error(f"[Lv1] {self.__class__.__name__} only accepts {self.field_type} values. "
                              f"but {expect_value!r} found.")

    def _func_count_should_be(self, value, expect_value):
        if isinstance(expect_value, int):
            if len(value) == expect_value:
                return value
            else:
                return self.error("[Lv2] %s count should be %s, but %s found."
                                  % (self.__class__.__name__, expect_value, len(value)))
        else:
            return self.error(f"[Lv2] `count_should_be` only accepts integer params, but {expect_value!r} found.")

    def _func_count_should_gt(self, value, expect_value):
        if isinstance(expect_value, int):
            if len(value) > expect_value:
                return value
            else:
                return self.error("[Lv2] %s count should greater than %s, but %s found."
                                  % (self.__class__.__name__, expect_value, len(value)))
        else:
            return self.error(f"[Lv2] `count_should_gt` only accepts integer params, but {expect_value!r} found.")

    def _func_count_should_gte(self, value, expect_value):
        if isinstance(expect_value, int):
            if len(value) >= expect_value:
                return value
            else:
                return self.error("[Lv2] %s count should greater than or equal to %s, but %s found."
                                  % (self.__class__.__name__, expect_value, len(value)))
        else:
            return self.error(f"[Lv2] `count_should_gte` only accepts integer params, but {expect_value!r} found.")

    def _func_count_should_lt(self, value, expect_value):
        if isinstance(expect_value, int):
            if len(value) < expect_value:
                return value
            else:
                return self.error("[Lv2] %s count should less than %s, but %s found."
                                  % (self.__class__.__name__, expect_value, len(value)))
        else:
            return self.error(f"[Lv2] `count_should_lt` only accepts integer params, but {expect_value!r} found.")

    def _func_count_should_lte(self, value, expect_value):
        if isinstance(expect_value, int):
            if len(value) <= expect_value:
                return value
            else:
                return self.error("[Lv2] %s count should less than or equal to %s, but %s found."
                                  % (self.__class__.__name__, expect_value, len(value)))
        else:
            return self.error(f"[Lv2] `count_should_lte` only accepts integer params, but {expect_value!r} found.")

    def _func_members_should_contain_model(self, value, expect_model):
        """It requires that members must contain a specific validator-model"""
        # Due to complications of circular imports, judge class by `__class__.__base__.__name__`.
        if expect_model.__class__.__base__.__name__ == 'ValidatorModel':
            for v in value:
                v = expect_model.validate(v)
                if self._validation_is_error(v):
                    return self.error(f"[Lv2] {self.__class__.__name__}'s members should contain {expect_model!r}, "
                                      f"but {v!r} does not match.")
            return value
        else:
            return self.error(f"[Lv2] `members_should_contain_model` only accepts ValidatorModel params, "
                              f"but {type(expect_model)!r} found.")

    def validate(self, value):
        # Lv1 validate
        # If validate fail, just return value.
        if self._validation_is_error(value):
            return value

        if self.field_type is not None:
            value = self._validate_types(value, self.field_type)

        # If validate fail, just return value.
        if self._validation_is_error(value):
            return value

        if len(self.fields) > len(value):
            return self.error(f"[Lv1] {self.__class__.__name__} expects {len(self.fields)} objects, but {len(value)} found.")

        for i, v in enumerate(self.fields):
            if issubclass(v.__class__, BaseField):
                v = v.validate(value[i])
            # Due to complications of circular imports, judge class by `__class__.__base__.__name__`.
            elif v.__class__.__base__.__name__ == 'ValidatorModel':
                v = v.validate(value[i])

            value[i] = v

        # Lv2 validate
        value = self._validate_should_funcs(value)

        return value


class DynamicListField(ListField):
    """This filed can be a little complicated.

    It validates every field and try to find a match result.
    We will loop through all elements to do this.
    This may take a long time."""

    __slots__ = ('should_be', 'should_in', 'should_contain', 'should_not_contain',
                 # For list field
                 'count_should_be', 'count_should_gt', 'count_should_lt',
                 'count_should_gte', 'count_should_lte', 'should_no_duplicates',
                 # For list members
                 'members_should_contain_model'
                 )

    field_type = list

    def _func_should_be(self, value: list, expect_value):
        if isinstance(expect_value, list):
            value.sort()
            expect_value.sort()
            if value == expect_value:
                return value
        return self.error("[Lv2] %s should be %s but %s found." % (self.__class__.__name__, expect_value, value))

    def validate(self, value):
        original_value = value
        # Lv1 validate
        # If validate fail, just return value.
        if self._validation_is_error(value):
            return value

        if self.field_type is not None:
            value = self._validate_types(value, self.field_type)

        # If validate fail, just return value.
        if self._validation_is_error(value):
            return value

        int_field_list = []
        str_field_list = []
        lst_field_list = []
        model_field_list = []

        int_value_list = []
        str_value_list = []
        lst_value_list = []
        dict_value_list = []
        # Categorize fields, to validate if field type match.
        for field in self.fields:
            if isinstance(field, IntegerField):
                int_field_list.append(field)
            elif isinstance(field, StringField):
                str_field_list.append(field)
            elif isinstance(field, ListField) or isinstance(field, DynamicListField):
                lst_field_list.append(field)
            # Due to complications of circular imports, judge class by `__class__.__base__.__name__`.
            elif field.__class__.__base__.__name__ == 'ValidatorModel':
                model_field_list.append(field)

        # Categorize values
        for v in value:
            if isinstance(v, int):
                int_value_list.append(v)
            elif isinstance(v, str):
                str_value_list.append(v)
            elif isinstance(v, list):
                lst_value_list.append(v)
            elif isinstance(v, dict):
                dict_value_list.append(v)

        # At first, the count of values list should always >= the count of fields list.
        if len(int_field_list) > len(int_value_list):
            return self.error(f"Expect {len(int_field_list)} integer objects, but {len(int_value_list)} found.")
        if len(str_field_list) > len(str_value_list):
            return self.error(f"Expect {len(str_field_list)} string objects, but {len(str_value_list)} found.")
        if len(lst_field_list) > len(lst_value_list):
            return self.error(f"Expect {len(lst_field_list)} list objects, but {len(lst_value_list)} found.")
        if len(model_field_list) > len(dict_value_list):
            return self.error(f"Expect {len(model_field_list)} dict objects, but {len(dict_value_list)} found.")

        # Then we validate all the type one by one.
        # A list to record validate result.
        validate_result = []
        for field in self.fields:
            value = 'not_assigned'
            if isinstance(field, IntegerField):
                value, int_value_list = self._validate_field_in_value_list(field, int_value_list)

            elif isinstance(field, StringField):
                value, str_value_list = self._validate_field_in_value_list(field, str_value_list)

            elif isinstance(field, ListField) or isinstance(field, DynamicListField):
                value, lst_value_list = self._validate_field_in_value_list(field, lst_value_list)

            elif field.__class__.__base__.__name__ == 'ValidatorModel':
                value, dict_value_list = self._validate_field_in_value_list(field, dict_value_list)

            if self._validation_is_error(value):
                validate_result.append(value)
                return validate_result

        # Lv2 validate, we need use original value to validate.
        validate_result = self._validate_should_funcs(original_value)
        return validate_result

    @staticmethod
    def _validate_field_in_value_list(field, value_list: list):
        value = None
        lv2_value = None
        for value in value_list:
            value = field.validate(value)
            # If validate pass, remove the value to avoid validation conflicts.
            if 'ValidationError' not in str(value):
                value_list.remove(value)
                return value, value_list
            else:
                # Lv2 error means the validation goes to final stage. we should take this error priority to Lv1.
                if '[Lv2]' in str(value):
                    lv2_value = value
        return lv2_value if lv2_value else value, value_list




# -*- coding: utf-8 -*-
"""
# @Time    : 2022/3/5 14:26
# @Author  : bruce
# @desc    :
This module contains the default values for all project used by ApiTestEz.
"""
import time

# LOG
CONSOLE_LOG_FORMAT = '%(asctime)s [%(levelname)s] [%(filename)s:%(lineno)d] [%(thread)d] - %(message)s'
CONSOLE_LOG_LEVEL = 'INFO'

FILE_LOG_FORMAT = '%(asctime)s [%(levelname)s] [%(filename)s:%(lineno)d] [%(thread)d] - %(message)s'
FILE_LOG_LEVEL = 'DEBUG'
FILE_LOG_PATH = None

# REPORT
REPORT_DIR = None
REPORT_STYLE = 'br'
REPORT_FILE_NAME = f'{time.strftime("%Y_%m_%d_%H_%M_%S", time.localtime(int(time.time())))}.html'
REPORT_TITLE = 'ApiTestEz Report'
REPORT_DESC = 'This is an api-test report generated by ApiTestEz.'
BR_REPORT_THEME = 'theme_default'



# -*- coding: utf-8 -*-
"""
# @Time    : 2022/3/3 15:41
# @Author  : bruce
# @desc    :
"""
from api_test_ez.ez.decorator.singleton import singleton
from api_test_ez.ez.decorator.jsonbean import json_bean
